// ESAF : Euso Simulation and Analysis Framework
// $Id: GlobalHisto.h,v 1.2 2005/01/06 00:24:57 thea Exp $
// A.Thea created May,  8 2003

#ifndef __GLOBALHISTO_HH_
#define __GLOBALHISTO_HH_

#if !defined (__CINT__) || defined (__MAKECINT__)
#include "TTree.h"
#include "TBranch.h"
#include "TArrayF.h"
#include "EEvent.hh"
#endif

class GlobalHisto {
public:
    GlobalHisto( TTree* );
    virtual ~GlobalHisto();

    TH1* Do1DNumPhotons();
    TH1* Do1DX();
    TH1* Do1DDetectedPhotonsFromChip();
    TH1* Do1DDetectedPhotonsFromMacrocells();
    TH1* Do1DPhotonsLostInOptics();
    TH1* Do1DPhotonsLostOnFocalPlane();
    TH1* Do1DHitPixels();
    TH1* Do1DHitMacroCells();
    TH1* Do1DNumOfFee();
    TH1* Do1DNumOfAFee();
    TH1* Do1DTriggerVsX();
    TH1* Do2DDetectedPhotonsVsX();
    TH1* Do2DAveragePhotonVsX();
    TH1* Do2DXCorrelation();

    Bool_t SetEnergyCut(Float_t EMin=0, Float_t EMax=0);
    Bool_t SetThetaOneCut(Float_t T1Min=0, Float_t T1Max=0);
    Bool_t SetThetaCut(Float_t TMin=0, Float_t TMax=0);
    Bool_t SetXVariable(Option_t*);
    inline TString* GetXVariable() {return &fCurrentX;}
    Bool_t IsInsideCuts(Int_t);
	
private:
   
    void Init(TTree *);

    TTree *fTree;
    EEvent *fEvent;
    Int_t fNumEvents;

    Float_t *fXVariable;
    Float_t fXMin;
    Float_t fXMax;
    TString fCurrentX;
    TString fXLabel;
    TString fXTitle;
       	

    // Global arrays 
    Int_t   *fGlobalNumPhotons; 
    Float_t *fGlobalTrueEnergy;
    Float_t *fGlobalTrueThetaOne;
    Float_t *fGlobalTrueTheta;

    Int_t fNoPhotons;

    // Extermes of the Global Arrays
    Int_t   fGlobalMaxNumPhotons;
    
    // Cuts
    Float_t fCutMinEnergy;
    Float_t fCutMaxEnergy;
    Float_t fCutMinThetaOne;
    Float_t fCutMaxThetaOne;
    Float_t fCutMinTheta;
    Float_t fCutMaxTheta;
};

#endif  /* __GLOBALHISTO_HH_ */

