#include "EsafMsgDispatcher.hh"
#include "EusoDetector.hh"
#include "TestLightToEuso.hh"
#include "DetectorTransportManager.hh"
#include "FocalPlane.hh"
#include "Photon.hh"
#include "PhotonsOnPupil.hh"
#include "Etypes.hh"
#include <TAxis3D.h>
#include <TGraph.h>
#include <TPolyMarker3D.h>
#include <TCanvas.h>
#include <TView.h>

EusoDetector* e = 0;

//______________________________________________________________________________
// 
// CheckPmtShift()
//
// Macro to check the dispacemente between the photomultipliers (pmts) and the
// ideal focal surface (ifs) using the photon tracking engine. A streak of
// light (0-30 deg) is transmitted through the optics and recorded on the focal
// plane.  FakeOpticalAdaptor's are fixed to have the pmts' photocathod exactly
// on the ifs. 
// 
// The photon's final position on the pmts and on the ifs are plotted in the
// focal plane's reference system in 2D and 3D.
// 

//______________________________________________________________________________
void CheckPmtShift() {

    Config* cfg = Config::Get();
    cfg->Reset();
    EsafMsgDispatcher::Get()->SetScreenSeverity(EsafMsg::Debug );

    // 
    cfg->AddUserDefinition("TestLightToEuso","TestLightToEuso.Type=EXTRADIUS");
    cfg->AddUserDefinition("TestLightToEuso", "TestLightToEuso.Duration=5");
    cfg->AddUserDefinition("TestLightToEuso", "TestLightToEuso.Photons=50000");
    cfg->AddUserDefinition("TestLightToEuso", "TestLightToEuso.Theta1=0");
    cfg->AddUserDefinition("TestLightToEuso", "TestLightToEuso.Theta2=30");
    cfg->AddUserDefinition("TestLightToEuso", "TestLightToEuso.Phi1=0");
    cfg->AddUserDefinition("OpticsFactory", "OpticsFactory.fOpticalAdaptor=FakeOpticalAdaptor");
    //   cfg->AddUserDefinition("TestLightToEuso.Phi2=1250");

    LightToEuso* lte = new TestLightToEuso;
    if ( !e ) e = new EusoDetector();
    
    DetectorTransportManager *dm = e->GetDetectorTransportManager();
    FocalPlane *fs = dm->GetFocalPlane();

    PhotonsOnPupil* pop = lte->Get(e->GetGeometry());
    e->Get(pop);
    cout << " Tracking Complete" << endl;

    pop->Rewind();
    Int_t n(0);
    Photon* p = 0;

    TGraph* gFS = new TGraph();
    TGraph* gIFS = new TGraph();

    gIFS->SetMarkerColor(kRed);

    TPolyMarker3D* mFS = new TPolyMarker3D();
    TPolyMarker3D* mIFS = new TPolyMarker3D();

    mIFS->SetMarkerColor(kRed);
    
    TVector3 pos;


    while ((p = pop->GetPhoton()))
        if ( p->hitIfs && p->history == kFocalPlane ) {

            pos = p->pos-fs->Position();
            gFS->SetPoint(n, pos.X(),pos.Z());
            mFS->SetPoint(n, pos.X(),pos.Y(),pos.Z());

            pos = p->posOnIfs-fs->Position();
            gIFS->SetPoint(n, pos.X(),pos.Z());
            mIFS->SetPoint(n, pos.X(),pos.Y(),pos.Z());
            n++;
        }
    
    cout <<  n << endl;
    
    if ( n ) { 
        TCanvas *c = new TCanvas("c_pmtshift","Pmt Shift");
        c->Divide(2,1);
        c->cd(1);
        gIFS->Draw("AP");
        gFS->Draw("P");
        TH1* h = gIFS->GetHistogram();
        h->SetXTitle("X [mm]");
        h->SetYTitle("Z [mm]");

        c->cd(2);

        mIFS->Draw();
        mFS->Draw("same");

        c->Modified();
        c->Update();
        
        TView* v = gPad->GetView();
        v->SetPerspective();
        TAxis3D::ToggleRulers();
    }



    
    delete lte;
}
