Float_t phi = 0;
gStyle->SetPalette(51);
TString dbname = "output/PopticsresponseDB_new.root";
//TString dbname = "/tmp/opticsresponseDB.root.save";
EOpticsResponse* Or = NULL;
Bool_t IsInitialized = kFALSE;

//______________________________________________________________________________
void opticsresponse()
{
   bar = new TControlBar("vertical", "Optics Response Viewer",10,10);
   
   bar->AddButton("How to run        ", "help()","Instructions for running this macro"); 
   bar->AddButton("Build DB          ", "orbuild()","Build the Optics Response DB"); 
   bar->AddButton("Eff vs Theta      ", "orread_theta()","Efficacies vs Theta");
   bar->AddButton("Eff vs Lambda     ", "orread_lambda()","Efficacies vs Lambda");
   bar->AddButton("Eff vs Phi        ", "orread_phi()","Efficacies vs Phi");
   bar->AddButton("Eff vs Phi.Anim.  ", "orread_phi_animated()","Efficacies vs Phi Animated");
   bar->AddButton("Eff vs Lambda. Histo", "orread_histo_lambda()","Efficacies vs Lambda.Histos");
   bar->AddButton("Eff for Spectrum.", "orread_mixed()","Efficacies for the spectrum. Just a test!");
    
   bar->Show();
   gROOT->SaveContext();
}
void help () {}
//______________________________________________________________________________
Bool_t init() {
    if (IsInitialized) return kTRUE;
    Or = new EOpticsResponse(dbname);
    IsInitialized = kTRUE;
    return kTRUE;
}
//______________________________________________________________________________
void orbuild() {
    ifs = new KIdealFocalSurface;
    os = new ParamOpticalSystem;
    OpticsResponseBuilder *orb = new OpticsResponseBuilder;
    orb->SetOpticalSystem(os);
    orb->SetIdealFocalSurface(ifs);
    orb->SetDBName(dbname);
    orb->SetNrays(1000000);
    orb->SetBins(1,2);
    orb->BuildEfficacyTables();
}

//______________________________________________________________________________
void orread_theta() {
    init();
    // display total and trigger efficacies as a function of theta
    Float_t binsTheta = 100, ThetaMax  = 32, stepTheta = ThetaMax/binsTheta; 
    TH1F *theta1 = new TH1F("theta1","Global Efficacy vs Theta",binsTheta,0,ThetaMax);
    TH1F *theta2 = new TH1F("theta2","Triggering Efficacy vs Theta",binsTheta,0,ThetaMax);
    theta1->SetDirectory(0);
    theta2->SetDirectory(0);
    TCanvas *c;
    if (! (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_eff_theta"))) {
        c = new TCanvas("c_eff_theta");
    }
    else
        c->Clear();
    Float_t theta, lambda_fixed = 3.6e-4; 	
    for (Int_t i(0); i < binsTheta; i++) {
         theta = (i+0.5)*stepTheta;
	 Or->GetEntry(theta*TMath::DegToRad(), lambda_fixed);
	 Float_t toteff  = Or->GetSample()->GetTotalEfficacy();
	 Float_t trigeff = Or->GetSample()->GetTriggEfficacy();
	 theta1->Fill(theta,toteff);
	 theta2->Fill(theta,trigeff);
    }
    theta1->Draw();
    theta2->Draw("same");
    theta1->GetXaxis()->SetTitle("Theta, [deg]");
    theta1->SetLineColor(kRed);
    theta2->SetLineColor(kBlue);	
    TLegend *legend=new TLegend(0.1,0.85,0.3,1);
    legend->SetTextFont(72);
    legend->SetTextSize(0.02);
    legend->AddEntry(theta1,"Total Efficacy","lp");
    legend->AddEntry(theta2,"Triggering Efficacy","l");
    legend->Draw();
    
}

//______________________________________________________________________________
void orread_lambda() {
    init();
    // display total and trigger efficacies as a function of lambda
    Float_t bins = 100, LambdaMax  = 400, LambdaMin=300, step = (LambdaMax-LambdaMin)/bins; 
    TH1F *theta1 = new TH1F("theta1","Global Efficacy vs Lambda",bins,LambdaMin,LambdaMax);
    TH1F *theta2 = new TH1F("theta2","Triggering Efficacy vs Lambda",bins,LambdaMin,LambdaMax);
    theta1->SetDirectory(0);
    theta2->SetDirectory(0);
    
    TCanvas *c;
    if (! (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_eff_lambda"))) {
        c = new TCanvas("c_eff_lambda");
    }
    else
        c->Clear();
    Float_t theta_fixed = 15*TMath::DegToRad(), lambda; 	
    for (Int_t i(0); i < bins; i++) {
         lambda = LambdaMin + (i+0.5)*step;
	 Or->GetEntry(theta_fixed, lambda*1.e-6);
	 Float_t toteff  = Or->GetSample()->GetTotalEfficacy();
	 Float_t trigeff = Or->GetSample()->GetTriggEfficacy();
	 theta1->Fill(lambda,toteff);
	 theta2->Fill(lambda,trigeff);
    }
    theta1->SetMinimum(0);
    theta1->Draw();
    theta2->Draw("same");
    theta1->GetXaxis()->SetTitle("Lambda, [nm]");
    theta1->SetLineColor(kRed);
    theta2->SetLineColor(kBlue);	
    TLegend *legend=new TLegend(0.1,0.85,0.3,1);
    legend->SetTextFont(72);
    legend->SetTextSize(0.02);
    legend->AddEntry(theta1,"Total Efficacy","lp");
    legend->AddEntry(theta2,"Triggering Efficacy","l");
    legend->Draw();
    
}

//______________________________________________________________________________
void orread_phi() {
    init();
    TCanvas *c;
    if (! (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_eff_phi"))) {
        c = new TCanvas("c_eff_phi");
    }
    else
        c->Clear();
    c->Divide(4,4);
    Float_t theta_fixed = 0*TMath::DegToRad(), lambda_fixed = 3.5e-4;
    Or->GetEntry(theta_fixed,lambda_fixed);
    Clear();
    TH2F* h[16];
    for (Int_t i = 0; i<16; i++) {
         c->cd(i+1);
	 gPad->AddExec("zoom","myzoom()");
	 Float_t phi = 360/16*(i+1);
	 h[i] = Or->GetSample()->MakePsfHistRotated(phi*TMath::DegToRad());
	 h[i]->SetName(Form("hRotated%d",i));
	 h[i]->Draw("col");
	 h[i]->GetXaxis()->SetTitle("mm");
         h[i]->GetYaxis()->SetTitle("mm");    
    }
   
}
//______________________________________________________________________________
void orread_phi_animated() {
    init();
    TCanvas *c;
    if (! (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_eff_phi_animated"))) {
        c = new TCanvas("c_eff_phi_animated");
    }
    else
        c->Clear();
    Float_t theta_fixed = 15*TMath::DegToRad(), lambda_fixed = 3.5e-4;
    Or->GetEntry(theta_fixed,lambda_fixed);
    TTimer *timer = new TTimer(20);
    timer->SetCommand("AnimateRotation()");
    timer->TurnOn();
   
}

//______________________________________________________________________________
void AnimateRotation() {
    TCanvas *c;
    if (!(c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_eff_phi_animated"))) return; 
    phi += 10; 
    if (phi >=360) phi -= 360;
    TH2F* h;
    Clear();
    h = Or->GetSample()->MakePsfHistRotated(phi*TMath::DegToRad());
    h->Draw("col");
    h->GetXaxis()->SetTitle("mm");
    h->GetYaxis()->SetTitle("mm");    
    c->Modified();
    c->Update();
    
}
  
//______________________________________________________________________________
void myzoom() {
   //this function is executed whenever the mouse is moved in a pad
   TPad * pad = gPad;
   TCanvas *big = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("big");
   if (!big) big = new TCanvas("big","big",600,10,600,600);
   big->cd();
   TIter next(pad->GetListOfPrimitives());
   while (obj=next())) {
      if (obj->InheritsFrom("TH1")) obj->Draw("colz");
   }
   big->Update();
}         
         
//______________________________________________________________________________
void orread_mixed() {
    init();
    TCanvas *c;
    if (! (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_eff_mixed"))) {
        c = new TCanvas("c_eff_mixed");
    }
    else
        c->Clear();
    c->Divide(1,2);
    TH1F* spec;
    if (spec = (TH1F*)gROOT->FindObject("spec")) {
        delete spec; 
        spec = NULL;
    }
    spec = new TH1F("spec","",100,3.e-4,4.e-4);
    
    EsafSpectrum *es = new EsafSpectrum();
    FluoCalculator *fFluoCalculator = LightSourceFactory::Get()->GetFluoCalculator( "nagano" );
    Float_t y = fFluoCalculator->GetFluoYield(10,80,es);
    Int_t stats(0);
    while (1) {
       stats++;
       Float_t lambda = es->GetLambda();
       spec->Fill(lambda);
       if (stats>1000) break;
    }
    c->cd(1);
    spec->Draw();
    spec->GetXaxis()->SetTitle("Lambda, [nm]");
    c->cd(2);
    TH2F* h;
    Clear();
    h = Or->MakePsfHist(10*TMath::DegToRad(),0*TMath::DegToRad(),spec);
    h->GetXaxis()->SetTitle("mm");
    h->GetYaxis()->SetTitle("mm");    
    h->Draw("colz");
    delete es;
}

//______________________________________________________________________________
void orread_histo_lambda() {
    init();
    TCanvas *c;
    if (! (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_eff_histo_lambda"))) {
        c = new TCanvas("c_eff_histo_lambda");
    }
    else
        c->Clear();
    c->Divide(4,4);
    Float_t theta = 15*TMath::DegToRad();
    Float_t phi   = 45*TMath::DegToRad(); 
    Clear();
    TH2F* histos[16];
    for (Int_t i = 0; i<16; i++) {
         c->cd(i+1);
	 gPad->AddExec("zoom","myzoom()");
	 Float_t lambda = 3.e-4 + 1.e-4.*i/16;
	 TH2F* h = Or->MakePsfHist(theta,phi,lambda);
	 histos[i] = (TH2F*)h->Clone();
	 histos[i]->SetName(Form("lambda_%d",i));
	 delete h;
 	 histos[i]->Draw("col");
	 histos[i]->GetXaxis()->SetTitle("mm");
         histos[i]->GetYaxis()->SetTitle("mm");    
   }
   delete [] histos;
}

//______________________________________________________________________________
void Clear() {
    TH2F* h;
    if (h=(TH2F*)gROOT->FindObject("hRotated")) {
        delete h;
    } 
}

