// ESAF : Euso Simulation and Analysis Framework
// $Id: RecoCellInfo.hh,v 1.8 2004/09/14 12:40:16 naumov Exp $
// Marco Pallavicini created Nov, 18 2003
// description of macrocell data
// each object of this kind describes data handled by a single macrocell
// 

#ifndef __RECOCELLINFO_HH_
#define __RECOCELLINFO_HH_

#include "euso.hh"
#include <vector>

class RecoCellHit;

class RecoCellInfo {
public:
    // ctor
    RecoCellInfo(Int_t cell);

    // dtor
    virtual ~RecoCellInfo();

    // get id
    inline Int_t GetCellId() const { return fCellId; }

    // trigger
    inline Bool_t HasTriggered() const { return fTriggered; }

    // trigger types
    inline Int_t TriggerWord() const { return fTriggerWord; }

    // get number of hits
    inline Int_t GetNumberofHits() const { return fHits.size(); }

    // return hits in 3 different modes
    
    // function
    const RecoCellHit* GetHit(UInt_t i) const;
    
    // operator
    inline const RecoCellHit* operator[] (Int_t i) const { return GetHit(i);}
    
    // reference to whole vector
    inline const vector<RecoCellHit*> &Hits() const { return fHits; }

    // return number of photoelectrons in the pixels of macrocell
    Int_t GetNumberofCounts() const;

    // return Gtu id of first photoelectron detected in macrocell
    Int_t GetFirstGtu() const;
    
    // return Gtu id of last photoelectron detected in macrocell
    Int_t GetLastGtu() const;
 

private:
    // cell id
    Int_t fCellId;
    
    // trigger flag
    Bool_t fTriggered;

    // trigger word
    Int_t fTriggerWord;

    // macrocell hits
    vector<RecoCellHit*> fHits;

    // add a hit; can only be done by root input module
    void AddHit(Int_t,Int_t,Int_t);

    friend class RootInputModule;

    ClassDef(RecoCellInfo,0)
};

#endif  /* __RECOCELLINFO_HH_ */

