// $Id: EnergyViewer.hh,v 1.18 2006/03/24 19:27:52 thea Exp $
// Author: Dmitry.Naumov   2005/09/25

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: EnergyViewer                                                         *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __ENERGYVIEWER_HH__
#define __ENERGYVIEWER_HH__

#include "euso.hh"
#include "EnergyTypes.hh"
#include "EsafMsgSource.hh"
#include <TString.h>
#include <TH2.h>
#include <TPaveText.h>
#include <TPolyLine.h>
#include <TPolyLine3D.h>

class TCanvas; 
class TGraphErrors;
class TGraph;
class TColor;
class TH1;
class TSpline3;

class EnergyModule;
class ERunParameters;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// EnergyViewer                                                               //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class EnergyViewer: public EsafMsgSource {
public:
    EnergyViewer(EnergyModule*);
    virtual ~EnergyViewer();

    void InitFilePS();
    void CloseFilePS();
    void DisplayPixels3D(TString DrawSurf);      // draw pixels in 3D
    void DisplayPixels();
    void DisplayFSTrack();
    void DisplayRecoveryDebug(TH1*,Float_t, Float_t);
    void DisplayRecovery();
    void DisplayCherenkov(TH1F *htime,Int_t nPoints, Float_t x[], Float_t y[]);
    void DisplayTimeFit(TH1F *htime, TF1 *func, TF1*);
    void DisplayShowerInfo();
    void DrawHisto(TH1F *histo);
    void DisplayAltitude(TH1F* histo, TF1* fit);
    void Clean();
private:

    void DefineColors();                          // define colors for pixels view
    void DefinePixelsView();                      // define pixels that will be drawn
    void DefineFSView();                          // 
    void SetHistoStyle(TH2F *histo);              // Set style for histo
    void DrawLabel(TPaveText *cl, TString pavetext); // Draw event information
    EnergyModule                 *fEnergyModule;  // pointer to the EnergyModule object
    map<Int_t,TPolyLine>          fMyPixelLines;  // map <Uid,TPolyLine*> for my pixels in cluster
    map<Int_t,TPolyLine>          fPixelLines;    // map <Uid,TPolyLine*> for pixels in cluster
    map<Int_t,TPolyLine3D>        fPixelLines3D;  // map <Uid,TPolyLine*> for pixels in cluster
    map<Int_t,TPolyLine>          fPmtLines;      // map <Uid,TPolyLine*> for pmts
    map<Int_t,TPolyLine3D>        fPmtLines3D;    // map <Uid,TPolyLine*> for pmts
    map<Int_t,TPolyLine3D>        fPmtLinesFS3D;  // map <Uid,TPolyLine*> for all pmts

    TCanvas                      *fCanvas;        // Canvas used for all debugging plots
    TString                       fPsFile;        // Debug Postscript file name
    TString                       fPsFileTime;    // Debug Postscript file name
    Float_t                       fXmin;          // Minimum X of View 
    Float_t                       fXmax;          // Maximum X of View
    Float_t                       fYmin;          // Minimum Y of View
    Float_t                       fYmax;          // Maximum Y of View
    Float_t                       fZmin;          // Minimum Z of View
    Float_t                       fZmax;          // Maximum Z of View
    Float_t                       fGXmin;         // Minimum X of View 
    Float_t                       fGXmax;         // Maximum X of View
    Float_t                       fGYmin;         // Minimum Y of View
    Float_t                       fGYmax;         // Maximum Y of View
    Float_t                       fGZmin;         // Minimum Z of View
    Float_t                       fGZmax;         // Maximum Z of View
    Int_t                         fColNum;        // number of used colors
    TColor                       *fColor;         // Color variable
    Bool_t                        fColorDefined;  // boolean: if the colors is already defined 
    Int_t                         fPalette[1000]; // Color palette

    Bool_t                        fFSViewBuild;   // if FSView is alredy built?
    Bool_t                        fDefinePixelsView; // if DefinePixelsView is alredy build?
    ClassDef(EnergyViewer,0)
};

#endif  /* __ENERGYVIEWER_HH__ */

