// $Id: RecoTreePainter.hh,v 1.6 2006/03/08 16:07:48 naumov Exp $
// Author: Dmitry.Naumov   2006/02/03

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoTreePainter                                                      *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __RECOTREEPAINTER_HH__
#define __RECOTREEPAINTER_HH__

#include "EsafMsgSource.hh"

#include <TObject.h>
#include <TString.h>
#include <TSystem.h>
#include <Riostream.h>
#include <map>

using std::map;
class TChain;
class RecoRootEvent;
class TH1;
class TList;
class TF1;
class TCanvas;
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//  RecoTreePainter                                                           //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class RecoTreePainter : public TObject, public EsafMsgSource {
public:
    RecoTreePainter(TChain*);
    virtual ~RecoTreePainter();
    void   Loop();
    void   DrawAll();
    TH1*   GetHisto(const char *name, const char *title, Option_t *opt, Int_t, Float_t, Float_t, Int_t, Float_t, Float_t);
    void   SetFileName(TString f)           { fFilename = f; }
    inline void   SaveEpsFiles(TString t);
private:
    void   FillHistos();
    void   SaveHistos();
    void   MakeBaseDataMap();
    Bool_t Accept();

    TChain              *fTree;
    RecoRootEvent       *fEvent;
    Int_t                fNevents;
    TList               *fHistoList;
    map <string,Float_t> fRec;
    map <string,Float_t> fSim;
    map <string,Float_t> left_edge;
    map <string,Float_t> right_edge;
    map <string,Bool_t>  fCuts;
    map <string,Bool_t>  fEnergyCuts;
    TString              fFilename;
    Bool_t               fSaveEpsFiles;
    TString              fEpsDir;

    ClassDef(RecoTreePainter,0)
};
inline void RecoTreePainter::SaveEpsFiles(TString t) {
    //
    //
    //
    fSaveEpsFiles = kTRUE;
    fEpsDir = t;
    gSystem->Exec(("mkdir -p "+t).Data());
}

#endif  /* __RECOTREEPAINTER_HH__ */

