// $Id: RecoProfile.hh,v 1.1 2005/11/23 16:02:40 moreggia Exp $
// Author: Sylvain Moreggia   2005/11/16

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoProfile                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __RECOPROFILE_HH__
#define __RECOPROFILE_HH__

#include <TObject.h>

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// RecoProfile                                                                 //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class RecoProfile : public TObject {
public:
    
    // ctor
    RecoProfile();
    
    // cpy ctor
    RecoProfile( const RecoProfile& );
    
    // dtor
    virtual ~RecoProfile();

    //copy to a new object
    virtual void Copy( TObject& ) const;

    //clear method
    void Clear();

    //setters
    inline void SetQuality(Double_t t)      {fQuality = t;}
    inline void SetXmax(Double_t t)         {fXmax = t;}
    inline void SetNemax(Double_t t)        {fNemax = t;}
    inline void SetSigma(Double_t t)        {fSigma = t;}
    inline void SetX1(Double_t t)           {fX1 = t;}
    inline void SetLambda(Double_t t)       {fLambda = t;}
    inline void SetErrorXmax(Double_t t)    {fErrorXmax = t;}
    inline void SetErrorNemax(Double_t t)   {fErrorNemax = t;}
    inline void SetErrorSigma(Double_t t)   {fErrorSigma = t;}
    inline void SetErrorX1(Double_t t)      {fErrorX1 = t;}
    inline void SetErrorLambda(Double_t t)  {fErrorLambda = t;}

    //getters
    inline Double_t GetQuality() const {return fQuality;}
    inline Double_t GetXmax()    const {return fXmax;}
    inline Double_t GetNemax()   const {return fNemax;}
    inline Double_t GetSigma()   const {return fSigma;}
    inline Double_t GetX1()      const {return fX1;}
    inline Double_t GetLambda()  const {return fLambda;}
    
private:
    
    Double_t fQuality;               // reconstruction quality variable
    Double_t fXmax;                  // reconstructed Xmax
    Double_t fNemax;                 // reconstructed Nmax
    Double_t fSigma;                 // reconstructed sigma if GFA
    Double_t fX1;                    // reconstructed X1 if GHF
    Double_t fLambda;                // reconstructed sigma if GHF
    Double_t fErrorXmax;             // related error
    Double_t fErrorNemax;            // related error
    Double_t fErrorSigma;            // related error
    Double_t fErrorX1;               // related error
    Double_t fErrorLambda;           // related error

    ClassDef(RecoProfile,1)
};

#endif  /* __RECOPROFILE_HH__ */

