// $Id: RecoTrackDirection.hh,v 1.5 2005/01/05 13:08:19 pesce Exp $
// Author: Roberto Pesce  2005/01/04

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoTrackDirection                                                   *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __RECOTRACKDIRECTION_HH__
#define __RECOTRACKDIRECTION_HH__

#include "TObject.h"
#include "TMultiGraph.h"


////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// RecoTrackDirection                                                         //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class RecoTrackDirection : public TObject{
public:
    RecoTrackDirection();
    //ctor
    
    RecoTrackDirection(const RecoTrackDirection&);
    //copy ctor

    virtual ~RecoTrackDirection();
    //dtor
    
    virtual void Copy( TObject& ) const;
    // copy to new object
    
    void Clear();
    // clear method
   
    // setters
    inline void SetQuality(Double_t t)        {fQuality = t;}
    inline void SetTheta(Double_t t)          {fTheta = t;}
    inline void SetErrorTheta(Double_t t)     {fErrorTheta = t;}
    inline void SetPhi(Double_t t)            {fPhi = t;}
    inline void SetErrorPhi(Double_t t)       {fErrorPhi = t;}
    inline void SetErrorDirection(Double_t t) {fErrorDirection = t;}
    void SetCheckInfo(TMultiGraph *t);
    
    // getters
    inline TMultiGraph* GetCheckInfo()      const {return fGraph;}
    inline Double_t     GetQuality()        const {return fQuality;}
    inline Double_t     GetTheta()          const {return fTheta;}
    inline Double_t     GetPhi()            const {return fPhi;}
    inline Double_t     GetErrorTheta()     const {return fErrorTheta;}
    inline Double_t     GetErrorPhi()       const {return fErrorPhi;}
    inline Double_t     GetErrorDirection() const {return fErrorDirection;}

private:
    Double_t fQuality;               // reconstruction quality variable defined by the the module responsible
    Double_t fTheta;                 // reconstructed Theta
    Double_t fPhi;                   // reconstructed Phi
    Double_t fErrorTheta;            // reconstruction error on Theta
    Double_t fErrorPhi;              // reconstruction error on Phi
    Double_t fErrorDirection;        // reconstruction error on the track direction
    TMultiGraph *fGraph;             //|| do not expand the object in the root tree

    ClassDef(RecoTrackDirection,1)
};

#endif  /* __RECOTRACKDIRECTION_HH__ */

