// $Id: EChipTriggParsFiller.cc,v 1.4 2005/11/17 09:11:46 pesce Exp $
// Author: R. Pesce   2005/10/21

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: EChipTriggParsFiller                                                 *
 *  Package: Electronics                                                     *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// EChipTriggParsFiller
//
// Interface class that fills the chip tracking trigger parameters
// in the rootfile. The filling of the parameters is called by each
// chip tracking trigger constructor.  
// 
//

#include "EChipTriggParsFiller.hh"
#include "ERunParameters.hh"
#include "EChipTriggPars.hh"

ClassImp(EChipTriggParsFiller)

//_____________________________________________________________________________
EChipTriggParsFiller::EChipTriggParsFiller() {
    //
    // Constructor
    //
    
    Clear();
}

//_____________________________________________________________________________
EChipTriggParsFiller::~EChipTriggParsFiller() {
    //
    // Destructor
    //
}

//_____________________________________________________________________________
void EChipTriggParsFiller::Clear() {
    //
    // Clear
    //

    fName = 0;
    fId = (ETriggerTypeIdentifier)0;
    fRelativeThreshold = kFALSE;
    fThreshold = -1;
    fMinTrackLength = -1;
    fMaxTrackLength = -1;
    fMinTriggerTrackLength = -1;
    fMinTriggerTwoLength = -1;
    fMaxTwoLength = -1;
    fOnlyWithSignal = kFALSE;
    fAcceptHole = kFALSE;
}

//_____________________________________________________________________________
void EChipTriggParsFiller::Fill(ERunParameters *runpars) {
    //
    // Filler
    //

    EChipTriggPars *ctp = new EChipTriggPars();

    ctp->SetName(fName);
    ctp->SetId(fId);
    ctp->SetHasRelativeThreshold(fRelativeThreshold);
    ctp->SetThreshold(fThreshold);
    ctp->SetMinTrackLength(fMinTrackLength);
    ctp->SetMaxTrackLength(fMaxTrackLength);
    ctp->SetMinTriggerTrackLength(fMinTriggerTrackLength);
    ctp->SetMinTriggerTwoLength(fMinTriggerTwoLength);
    ctp->SetMaxTwoLength(fMaxTwoLength);
    ctp->SetAcceptHole(fAcceptHole);
    ctp->SetOnlyWithSignal(fOnlyWithSignal);

    if (!runpars->fTriggPars) runpars->fTriggPars = new TList();
    (runpars->fTriggPars)->Add(ctp);

}

