// ESAF : Euso Simulation and Analysis Framework
// $Id: FocalSurfaceFileParser.cc,v 1.4 2006/03/01 20:54:01 thea Exp $
// M. Pallavicini - created

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: FocalSurfaceFileParser                                               *
 *  Package: FocalSurfaceFileParser                                          *
 *  Coordinator: Marco.Pallavicini                                           *
 *                                                                           *
 *****************************************************************************/

#include "FocalSurfaceFileParser.hh"
        
ClassImp(FocalSurfaceFileParser)

// number of columns in focal surface file
#define FSFCOLUMNS 17

//______________________________________________________________________________
FocalSurfaceFileParser::FocalSurfaceFileParser( string& fn ) 
  : NumbersFileParser( fn, FSFCOLUMNS ) {
    //
    // Constructor
    //
    fCurrent = -1;
    fOldPmt = 0;
    fOldMC = 0;
    fOldEC = 0;
    fOldChip = 0;
}

//______________________________________________________________________________
FocalSurfaceFileParser::~FocalSurfaceFileParser() {
    // 
    // Destructor
    // 
}
   
//______________________________________________________________________________
Int_t FocalSurfaceFileParser::NewPmtId() {
    //
    // Returns PMT number of the current line at each call move one line down
    // returns 0 when number table is over
    //
    fCurrent++;
    if ( fCurrent >= GetNumLines() ) {
        return 0;
    }
    Int_t n = (Int_t) GetNum( fCurrent, PmtIdCol );
    if ( n != (fOldPmt+1) ) {
        cerr << "Old = " << fOldPmt << " New=" << n << "  Lines=" << GetNumLines() << endl;
        throw runtime_error("FocalSurfaceFileParser: Format error in NewPmtId()");
    }
    fOldPmt = n;
    return n;
}

//______________________________________________________________________________
Int_t FocalSurfaceFileParser::NewMacroCell() {
    //
    // Return macrocell id if we have a new one, otherwise 0
    //
    Int_t n = (Int_t) GetNum( fCurrent, MCIdCol );
    if ( n == fOldMC ) {
        return 0;
    }
    else if ( n == (fOldMC+1) ) {
        fOldMC = n;
	return n;
    } else {
        cerr << "Old = " << fOldMC << " New=" << n << endl;
        throw runtime_error("FocalSurfaceFileParser: Format error in NewMacroCell()");
    }
}

//______________________________________________________________________________
Int_t FocalSurfaceFileParser::NewElementaryCell() {
    //
    // Return elementary cell id if we have a new one, otherwise 0
    //
    Int_t n = (Int_t) GetNum( fCurrent, ECIdCol );
    if ( n == fOldEC ) {
        return 0;
    }
    else if ( n == (fOldEC+1) ) {
        fOldEC = n;
	return n;
    } else {
        cerr << "Old = " << fOldEC << " New=" << n << endl;
        throw runtime_error("FocalSurfaceFileParser: Format error NewElementaryCell()");
    }
}

//______________________________________________________________________________
Int_t FocalSurfaceFileParser::NewFrontEndChip() {
    // 
    // Return front end chip id if we have a new one, otherwise 0
    // 
    Int_t n = (Int_t) GetNum( fCurrent, ChipIdCol );
    if ( n == fOldChip ) {
        return 0;
    }
    else if ( n == (fOldChip+1) ) {
        fOldChip = n;
	return n;
    } else {
        cerr << "Old= " << fOldChip << " New=" << n << endl;
        throw runtime_error("FocalSurfaceFileParser: Format error in NewFrontEndChip()");
    }
}

//______________________________________________________________________________
const EVector& FocalSurfaceFileParser::Pos() {
    // 
    // Geometry parameters: position 
    // 
    fPos[0] = GetNum( fCurrent, PosXCol );
    fPos[1] = GetNum( fCurrent, PosYCol );
    fPos[2] = GetNum( fCurrent, PosZCol );
    return fPos;
}

//______________________________________________________________________________
const EVector& FocalSurfaceFileParser::Norm() {
    //
    // Geometry parameters: normal
    // 
    fNorm[0] = GetNum( fCurrent, NormXCol );
    fNorm[1] = GetNum( fCurrent, NormYCol );
    fNorm[2] = GetNum( fCurrent, NormZCol );
    return fNorm;
}

//______________________________________________________________________________
const EVector& FocalSurfaceFileParser::Dir() {
    //
    // Geometry parameters: direction 
    // 
    fDir[0] = GetNum( fCurrent, DirXCol );
    fDir[1] = GetNum( fCurrent, DirYCol );
    fDir[2] = GetNum( fCurrent, DirZCol );
    return fDir;
}

