// ESAF : Euso Simulation and Analysis Framework
// $Id: PhotonsDatabaseBuilder.hh,v 1.6 2004/09/28 12:10:30 thea Exp $
// A.Thea, J.Watts created Mar, 19 2004

#ifndef __PHOTONSDATABASEBUILDER_HH_
#define __PHOTONSDATABASEBUILDER_HH_

#include "euso.hh"
#include "EsafConfigurable.hh"
#include "ScanPhotonsOnPupil.hh"
#include "OADBTree.hh"
#include "OADBPhotons.hh"
#include "TFile.h"

class OpticalSystem;

class PhotonsDatabaseBuilder: public EsafConfigurable {
public:
    // ctor
    PhotonsDatabaseBuilder();

    // dtor
    virtual ~PhotonsDatabaseBuilder();

    // build the database
    void Go();

    EsafConfigClass(Tools,PhotonsDatabaseBuilder)
private:
     
    Bool_t Sample( Photon* );           // resamples a photon fSamples time
    
    void OpenRoot();                    // open rootfile
    void CloseRoot();                   // close rootfile    
    void InitDBTree();                  // create tree and branches
    void FillDBTree();                  // fill tree and clean event
    void CloseDBTree();                 // write tree, delete event

    ScanPhotonsOnPupil fPupil;          // photons generator
    pair<Photon, Double_t> fLastPath;   // 
    Int_t fSamples;                     // number of times each position has to be sampled

    OpticalSystem *fOptics;             // EUSO Optical System
    OADBTree *fTree;                    // database tree
    OADBPhotons *fPhotons;              // photons container
    string fRootFileName;
    TFile *fRootFile;
    
    ClassDef(PhotonsDatabaseBuilder,0)

};

#endif  /* __PHOTONSDATABASEBUILDER_HH_ */

