      SUBROUTINE ESAFLWTRANS(INIT,USR_ALT,USR_ANGLE,TRANS)
c**********************************************************************
c     Subroutine to calcul the radiative transfert throw the atmosphere
c     Based on the LOWTRAN7 program
c
c     Input parameters :
c     ------------------
c     INIT :  flag for initialisation of lowtran
C             0 initialisation of atmosphere's parameters and conditions 
c                 ( defined in the lowtran.input file )
c             1 initialisation of geometrical conditions only when
c               atmosphere's parameters have already been defined 
c
c     USR_ALT, USR_ANGLE : altitude of the point in km
c                          angle theta compare to Nadir (0=vertical)
c                        origine on ground at Nadir (EUSO = 0.,0.,400.)
c
c     Output parameters :
c     -------------------
c     TRANS(lambda) : total transmission from a point at alt usr_alt
c     to a point above the atmosphere in the theta direction
c     for differents wavelenghts (no more than 100 and defined in the
c     lowtran.input file)
c     Then, the transmission can be multiplied.
c
c     Author : Anne Stutz
c     Last revised by Corinne Berat, August 2002
c
c***********************************************************************
#include "lwtrn7.common"
#include "userlun.common"
#include "tuple.common"

      INTEGER INIT
      REAL USR_ALT,USR_ANGLE
      REAL TRANS(100)
      IFILE  = 8
      ITAPE5 = 9
      IPRNT  = 10
      IFILE2 = 18
      
c     Atmosphere Model Initialisation
      IF(INIT.EQ.0) THEN
         NEVT=1
         ILAMBDA=0
         CALL INITCARD3(0.,0.)
         CALL LWTRN7SUB
         DO I=1,100
            TRANS(I)=TRANSM(I)
         ENDDO   
      ENDIF   
         
c     Calcul the atmospheric transmission from xpos,ypos,zpos to EUSO 
      IF(INIT.EQ.1) THEN
         NEVT = NEVT+1
         ILAMBDA=0
         CALL INITCARD3(USR_ALT,USR_ANGLE)
         CALL LWTRN7SUB
         DO I=1,100
            TRANS(I) = TRANSM(I)
         ENDDO   
      ENDIF
c
      RETURN
      END


         









